@@ -7,9 +7,14 @@ use Carp;
 
 use Inline (
         C => "DATA",
-        LIBS => '-lusb',
-	NAME => 'Device::USB',
-	VERSION => '0.17',
+        ($ENV{LIBUSB_LIBDIR}
+            ? ( LIBS => "-L$ENV{LIBUSB_LIBDIR} " .
+                        ($^O eq 'MSWin32' ? '-llibusb' : '-lusb') )
+            : ( LIBS => '-lusb', )
+        ),
+        ($ENV{LIBUSB_INCDIR} ? ( INC => "-I$ENV{LIBUSB_INCDIR}" ) : () ),
+        NAME => 'Device::USB',
+        VERSION => '0.22',
    );
 
 Inline->init();
@@ -19,19 +24,32 @@ Inline->init();
 #
 
 use Device::USB::Device;
+use Device::USB::DevConfig;
+use Device::USB::DevInterface;
+use Device::USB::DevEndpoint;
 use Device::USB::Bus;
 
+use constant CLASS_PER_INSTANCE => 0;
+use constant CLASS_AUDIO => 1;
+use constant CLASS_COMM =>  2;
+use constant CLASS_HID =>   3;
+use constant CLASS_PRINTER => 7;
+use constant CLASS_MASS_STORAGE => 8;
+use constant CLASS_HUB =>     9;
+use constant CLASS_DATA =>   10;
+use constant CLASS_VENDOR_SPEC => 0xff;
+
 =head1 NAME
 
 Device::USB - Use libusb to access USB devices.
 
 =head1 VERSION
 
-Version 0.17
+Version 0.22
 
 =cut
 
-our $VERSION='0.17';
+our $VERSION='0.22';
 
 
 =head1 SYNOPSIS
@@ -98,6 +116,51 @@ functions of the libusb library. In particular, it provides interfaces to
 find busses and devices. It also provides convenience methods that simplify
 some of the tasks above.
 
+=head2 CONSTANTS
+
+This class provides a set of constants for the defined device classes. The
+constants defined at this time are:
+
+=over 4
+
+=item *
+
+CLASS_PER_INSTANCE
+
+=item *
+
+CLASS_AUDIO
+
+=item *
+
+CLASS_COMM
+
+=item *
+
+CLASS_HID
+
+=item *
+
+CLASS_PRINTER
+
+=item *
+
+CLASS_MASS_STORAGE
+
+=item *
+
+CLASS_HUB
+
+=item *
+
+CLASS_DATA
+
+=item *
+
+CLASS_VENDOR_SPEC
+
+=back
+
 =head2 FUNCTIONS
 
 =over 4
@@ -135,7 +198,7 @@ interface code.
 
 =over 4
 
-=item $level
+=item level
 
 0 disables debugging, 1 enables some debug messages, and 2 enables verbose
 debug messages
@@ -206,7 +269,9 @@ returns a device reference or undef if none was found.
 sub find_device
 {
     my $self = shift;
-    my ($vendor, $product) = @_;
+    my $vendor = shift;
+    my $product = shift;
+
     return lib_find_usb_device( $vendor, $product );
 }
 
@@ -239,14 +304,10 @@ sub find_device_if
     croak( "Predicate must be a code reference.\n" )
         unless 'CODE' eq ref $pred;
 
-    local $_;
-
     foreach my $bus ($self->list_busses())
     {
-        foreach($bus->devices())
-        {
-            return $_ if $pred->();
-        }
+        my $dev = $bus->find_device_if( $pred );
+        return $dev if defined $dev;
     }
 
     return;
@@ -279,7 +340,8 @@ to that array in scalar context
 sub list_devices
 {
     my $self = shift;
-    my ($vendor, $product) = @_;
+    my $vendor = shift;
+    my $product = shift;
     my $pred = undef;
 
     if(!defined $vendor)
@@ -317,12 +379,11 @@ coderef to test devices.
 For example,
 
     my @devices = $usb->list_devices_if(
-        sub { 9 == $_->bDeviceClass() }
+        sub { Device::USB::CLASS_HUB == $_->bDeviceClass() }
     );
 
-Returns all USB hubs found, because the device class for hub is 9. The
-device to test is available to the coderef in the C<$_> variable for
-simplicity.
+Returns all USB hubs found. The device to test is available to the coderef
+in the C<$_> variable for simplicity.
 
 =cut
 
@@ -338,12 +399,12 @@ sub list_devices_if
         unless 'CODE' eq ref $pred;
 
     my @devices = ();
-    local $_;
+    local $_ = undef;
 
     foreach my $bus ($self->list_busses())
     {
         # Push all matching devices for this bus on list.
-        push @devices, grep { $pred->() } $bus->devices();
+        push @devices, $bus->list_devices_if( $pred );
     }
 
     return wantarray ? @devices : \@devices;
@@ -423,19 +484,23 @@ compiling the module on Windows. In theory, it should be possible to make
 the library work with LibUsb-Win32 L<http://libusb-win32.sourceforge.net/>.
 Without access to a Windows development system, I can't make those changes.
 
+The Interfaces and Endpoints are not yet proper objects. The code to extract
+this information is not yet written.
+
 =head1 ACKNOWLEDGEMENTS
 
 Thanks go to various members of the Houston Perl Mongers group for input
 on the module. But thanks mostly go to Paul Archer who proposed the project
 and helped with the development.
 
-Thanks to Josep Monés Teixidor for fixing the \C<bInterfaceClass> bug.
-Thanks to Mike McCauley for support of \C<usb_get_driver_np> and
-\C<usb_detach_kernel_driver_np>.
+Thanks to Josep Monés Teixidor for fixing the C<bInterfaceClass> bug.
+Thanks to Mike McCauley for support of C<usb_get_driver_np> and
+C<usb_detach_kernel_driver_np>. Thanks to Vadim Mikhailov for fixing
+a compile problem with VC6 on Windows.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2006 Houston Perl Mongers
+Copyright 2006-2007 Houston Perl Mongers
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -516,7 +581,7 @@ int libusb_reset(void *dev)
 
 int libusb_get_driver_np(void *dev, int interface, char *name, unsigned int namelen)
 {
-    int ret;
+    int ret = 0;
     if(debugLevel)
     {
         printf( "libusb_get_driver_np( %d )\n", interface );
@@ -564,22 +629,23 @@ int libusb_release_interface(void *dev, int interface)
 void libusb_control_msg(void *dev, int requesttype, int request, int value, int index, char *bytes, int size, int timeout)
 {
     int i = 0;
+    int retval = 0;
+
+    Inline_Stack_Vars;
 
     if(debugLevel)
     {
         printf( "libusb_control_msg( %#04x, %#04x, %#04x, %#04x, %p, %d, %d )\n",
             requesttype, request, value, index, bytes, size, timeout
         );
-	/* maybe need to add support for printing the bytes string. */
+        /* maybe need to add support for printing the bytes string. */
     }
-    int retval = usb_control_msg((usb_dev_handle *)dev, requesttype, request, value, index, bytes, size, timeout);
+    retval = usb_control_msg((usb_dev_handle *)dev, requesttype, request, value, index, bytes, size, timeout);
     if(debugLevel)
     {
         printf( "\t => %d\n",retval );
     }
 
-    Inline_Stack_Vars;
-
     /* quiet compiler warnings. */
     (void)i;
     (void)ax;
@@ -607,8 +673,8 @@ int libusb_get_string(void *dev, int index, int langid, char *buf, size_t buflen
     if(debugLevel)
     {
         printf( "libusb_get_string( %d, %d, %p, %u )\n",
-	    index, langid, buf, buflen
-	);
+            index, langid, buf, buflen
+        );
     }
     return usb_get_string((usb_dev_handle *)dev, index, langid, buf, buflen);
 }
@@ -618,8 +684,8 @@ int libusb_get_string_simple(void *dev, int index, char *buf, size_t buflen)
     if(debugLevel)
     {
         printf( "libusb_get_string_simple( %d, %p, %u )\n",
-	    index, buf, buflen
-	);
+            index, buf, buflen
+        );
     }
     return usb_get_string_simple((usb_dev_handle *)dev, index, buf, buflen);
 }
@@ -726,6 +792,27 @@ static SV* build_descriptor(struct usb_device *dev)
     return newRV_noinc( (SV*)hash );
 }
 
+/*
+ * Given a pointer to a usb_endpoint_descriptor struct, create a reference
+ * to a Device::USB::DevEndpoint object that represents it.
+ */
+static SV* build_endpoint( struct usb_endpoint_descriptor* endpt )
+{
+    HV* hash = newHV();
+
+    hashStoreInt( hash, "bDescriptorType", endpt->bDescriptorType );
+    hashStoreInt( hash, "bEndpointAddress", endpt->bEndpointAddress );
+    hashStoreInt( hash, "bmAttributes", endpt->bmAttributes );
+    hashStoreInt( hash, "wMaxPacketSize", endpt->wMaxPacketSize );
+    hashStoreInt( hash, "bInterval", endpt->bInterval );
+    hashStoreInt( hash, "bRefresh", endpt->bRefresh );
+    hashStoreInt( hash, "bSynchAddress", endpt->bSynchAddress );
+
+    return sv_bless( newRV_noinc( (SV*)hash ),
+        gv_stashpv( "Device::USB::DevEndpoint", 1 )
+    );
+}
+
 /*
  * Given a pointer to an array of usb_endpoint_descriptor structs, create a
  * reference to a Perl array containing the same data.
@@ -733,34 +820,32 @@ static SV* build_descriptor(struct usb_device *dev)
 static SV* list_endpoints( struct usb_endpoint_descriptor* endpt, unsigned count )
 {
     AV* array = newAV();
-    HV* hash = 0;
-    unsigned i= 0;
+    unsigned i = 0;
 
     for(i=0; i < count; ++i)
     {
-        av_push( array, newRV_noinc( (SV*)(hash = newHV()) ) );
-        hashStoreInt( hash, "bDescriptorType", endpt[i].bDescriptorType );
-        hashStoreInt( hash, "bEndpointAddress", endpt[i].bEndpointAddress );
-        hashStoreInt( hash, "bmAttributes", endpt[i].bmAttributes );
-        hashStoreInt( hash, "wMaxPacketSize ", endpt[i].wMaxPacketSize );
-        hashStoreInt( hash, "bInterval", endpt[i].bInterval );
-        hashStoreInt( hash, "bRefresh", endpt[i].bRefresh );
-        hashStoreInt( hash, "bSynchAddress", endpt[i].bSynchAddress );
+        av_push( array, build_endpoint( endpt+i ) );
     }
 
     return newRV_noinc( (SV*)array );
 }
 
+
 /*
- * Given a pointer to a usb_interface_descriptor, copy the data into the
- * supplied hash.
+ * Build the object that contains the interface descriptor.
+ *
+ * inter - the usb_interface_descriptor describing this interface.
+ *
+ * returns the appropriate pointer to a reference.
  */
-static void store_interface( HV* hash, struct usb_interface_descriptor* inter )
+static SV* build_interface( struct usb_interface_descriptor* inter )
 {
+    HV* hash = newHV();
+
     hashStoreInt( hash, "bDescriptorType", inter->bDescriptorType );
     hashStoreInt( hash, "bInterfaceNumber", inter->bInterfaceNumber );
     hashStoreInt( hash, "bAlternateSetting", inter->bAlternateSetting );
-    hashStoreInt( hash, "bNumEndpoints ", inter->bNumEndpoints );
+    hashStoreInt( hash, "bNumEndpoints", inter->bNumEndpoints );
     hashStoreInt( hash, "bInterfaceClass", inter->bInterfaceClass );
     hashStoreInt( hash, "bInterfaceSubClass", inter->bInterfaceSubClass );
     hashStoreInt( hash, "bInterfaceProtocol", inter->bInterfaceProtocol );
@@ -768,7 +853,12 @@ static void store_interface( HV* hash, struct usb_interface_descriptor* inter )
     hashStoreSV( hash, "endpoints",
         list_endpoints( inter->endpoint, inter->bNumEndpoints )
     );
-}
+    /* TODO: handle the 'extra' data */
+
+    return sv_bless( newRV_noinc( (SV*)hash ),
+        gv_stashpv( "Device::USB::DevInterface", 1 )
+    );
+} 
 
 /*
  * Given a pointer to an array of usb_interface structs, create a
@@ -777,19 +867,44 @@ static void store_interface( HV* hash, struct usb_interface_descriptor* inter )
 static SV* list_interfaces( struct usb_interface* ints, unsigned count )
 {
     AV* array = newAV();
-    HV* hash = 0;
-    unsigned i= 0;
+    unsigned i = 0;
 
     for(i=0; i < count; ++i)
     {
-        av_push( array, newRV_noinc( (SV*)(hash = newHV()) ) );
-        hashStoreInt( hash, "num_altsetting", ints[i].num_altsetting );
-	store_interface( hash, ints[i].altsetting );
+        AV* inters = newAV();
+        unsigned j = 0;
+        for(j=0; j < ints[i].num_altsetting; ++j)
+        {
+            av_push( inters, build_interface( (ints[i].altsetting+j) ) );
+        }
+        av_push( array, newRV_noinc( (SV*)inters ) );
     }
 
     return newRV_noinc( (SV*)array );
 }
 
+/*
+ * Given a pointer to a usb_config_descriptor struct, create a Perl
+ * object that contains the same data.
+ */
+static SV* build_configuration( struct usb_config_descriptor *cfg )
+{
+    HV* hash = newHV();
+    hashStoreInt( hash, "bDescriptorType", cfg->bDescriptorType );
+    hashStoreInt( hash, "wTotalLength", cfg->wTotalLength );
+    hashStoreInt( hash, "bNumInterfaces", cfg->bNumInterfaces );
+    hashStoreInt( hash, "bConfigurationValue", cfg->bConfigurationValue );
+    hashStoreInt( hash, "iConfiguration", cfg->iConfiguration );
+    hashStoreInt( hash, "bmAttributes", cfg->bmAttributes );
+    hashStoreInt( hash, "MaxPower", cfg->MaxPower*2 );
+    hashStoreSV( hash, "interfaces",
+        list_interfaces( cfg->interface, cfg->bNumInterfaces )
+    );
+
+    return sv_bless( newRV_noinc( (SV*)hash ),
+        gv_stashpv( "Device::USB::DevConfig", 1 )
+    );
+}
 
 /*
  * Given a pointer to an array of usb_config_descriptor structs, create a
@@ -798,22 +913,11 @@ static SV* list_interfaces( struct usb_interface* ints, unsigned count )
 static SV* list_configurations(struct usb_config_descriptor *cfg, unsigned count )
 {
     AV* array = newAV();
-    HV* hash = 0;
-    unsigned i= 0;
+    unsigned i = 0;
 
     for(i=0; i < count; ++i)
     {
-        av_push( array, newRV_noinc( (SV*)(hash = newHV()) ) );
-        hashStoreInt( hash, "bDescriptorType", cfg[i].bDescriptorType );
-        hashStoreInt( hash, "wTotalLength", cfg[i].wTotalLength );
-        hashStoreInt( hash, "bNumInterfaces", cfg[i].bNumInterfaces );
-        hashStoreInt( hash, "bConfigurationValue", cfg[i].bConfigurationValue );
-        hashStoreInt( hash, "iConfiguration", cfg[i].iConfiguration );
-        hashStoreInt( hash, "bmAttributes", cfg[i].bmAttributes );
-        hashStoreInt( hash, "MaxPower", cfg[i].MaxPower );
-	hashStoreSV( hash, "interfaces",
-	    list_interfaces( cfg[i].interface, cfg[i].bNumInterfaces )
-	);
+        av_push( array, build_configuration( (cfg+i) ) );
     }
 
     return newRV_noinc( (SV*)array );
@@ -924,13 +1028,13 @@ SV *lib_find_usb_device( int vendor, int product )
     {
         struct usb_device *dev = 0;
         for(dev = bus->devices; 0 != dev; dev = dev->next)
-	{
+        {
             if((dev->descriptor.idVendor == vendor) &&
-	      (dev->descriptor.idProduct == product))
-	    {
+              (dev->descriptor.idProduct == product))
+            {
                 return build_device( dev );
             }
-	}
+        }
     }
 
     return &PL_sv_undef;
